@@ -6,7 +6,7 @@
  *	@author		Masaki Fujimoto <fujimoto@php.net>
  *	@license	http://www.opensource.org/licenses/bsd-license.php The BSD License
  *	@package	Ethna
- *	@version	$Id: Ethna.php,v 1.5 2004/12/09 05:35:21 fujimoto Exp $
+ *	@version	$Id: Ethna.php,v 1.11 2004/12/23 06:05:42 fujimoto Exp $
  */
 
 /** Ethna¸饤֥: PEAR, PEAR_Error */
@@ -20,7 +20,7 @@ include_once('Smarty/Smarty.class.php');
 
 
 /** С */
-define('ETHNA_VERSION', '0.1.0');
+define('ETHNA_VERSION', '0.1.2');
 
 /** Ethna١ǥ쥯ȥ */
 define('ETHNA_BASE', dirname(__FILE__));
@@ -38,7 +38,6 @@ include_once(ETHNA_BASE . '/class/Ethna_Controller.php');
 include_once(ETHNA_BASE . '/class/Ethna_DB.php');
 include_once(ETHNA_BASE . '/class/Ethna_Filter.php');
 include_once(ETHNA_BASE . '/class/Ethna_I18N.php');
-include_once(ETHNA_BASE . '/class/Ethna_InfoManager.php');
 include_once(ETHNA_BASE . '/class/Ethna_LogWriter.php');
 include_once(ETHNA_BASE . '/class/Ethna_LogWriter_File.php');
 include_once(ETHNA_BASE . '/class/Ethna_LogWriter_Syslog.php');
@@ -50,9 +49,7 @@ include_once(ETHNA_BASE . '/class/Ethna_SmartyPlugin.php');
 include_once(ETHNA_BASE . '/class/Ethna_Util.php');
 include_once(ETHNA_BASE . '/class/Ethna_ViewClass.php');
 include_once(ETHNA_BASE . '/class/AMF/Ethna_AMF_ActionClass.php');
-include_once(ETHNA_BASE . '/class/Action/Ethna_Action_Info.php');
 include_once(ETHNA_BASE . '/class/CLI/Ethna_CLI_ActionClass.php');
-include_once(ETHNA_BASE . '/class/Form/Ethna_Form_Info.php');
 include_once(ETHNA_BASE . '/class/View/Ethna_View_List.php');
 
 if (extension_loaded('soap')) {
@@ -97,19 +94,19 @@ define('DB_TYPE_MISC', 3);
 define('VAR_TYPE_INT', 1);
 
 /** Ƿ: ư */
-define('VAR_TYPE_FLOAT', 1);
+define('VAR_TYPE_FLOAT', 2);
 
 /** Ƿ: ʸ */
-define('VAR_TYPE_STRING', 2);
+define('VAR_TYPE_STRING', 3);
 
 /** Ƿ:  */
-define('VAR_TYPE_DATETIME', 3);
+define('VAR_TYPE_DATETIME', 4);
 
 /** Ƿ:  */
-define('VAR_TYPE_BOOLEAN', 4);
+define('VAR_TYPE_BOOLEAN', 5);
 
 /** Ƿ: ե */
-define('VAR_TYPE_FILE', 5);
+define('VAR_TYPE_FILE', 6);
 
 
 /** ե෿: text */
