@@ -6,7 +6,7 @@
  *	@author		Masaki Fujimoto <fujimoto@php.net>
  *	@license	http://www.opensource.org/licenses/bsd-license.php The BSD License
  *	@package	Ethna
- *	@version	$Id: Ethna_ActionForm.php,v 1.8 2004/12/08 16:22:30 fujimoto Exp $
+ *	@version	$Id: Ethna_ActionForm.php,v 1.9 2004/12/13 15:08:38 fujimoto Exp $
  */
 
 /** 귿ե륿: Ⱦ */
@@ -51,12 +51,12 @@ class Ethna_ActionForm
 	var $app_ne_vars = array();
 
 	/**
-	 *	@var	object	Ethna_ActionError	action error֥
+	 *	@var	object	Ethna_ActionError	󥨥顼֥
 	 */
 	var $action_error;
 
 	/**
-	 *	@var	object	Ethna_ActionError	action error֥(ά)
+	 *	@var	object	Ethna_ActionError	󥨥顼֥(ά)
 	 */
 	var $ae;
 
