@@ -6,7 +6,7 @@
  *	@author		Masaki Fujimoto <fujimoto@php.net>
  *	@license	http://www.opensource.org/licenses/bsd-license.php The BSD License
  *	@package	Ethna
- *	@version	$Id: Ethna_AppObject.php,v 1.4 2004/12/09 03:14:00 fujimoto Exp $
+ *	@version	$Id: Ethna_AppObject.php,v 1.5 2004/12/13 15:08:38 fujimoto Exp $
  */
 
 // {{{ Ethna_AppObject
@@ -45,12 +45,12 @@ class Ethna_AppObject
 	var $i18n;
 
 	/**
-	 *	@var	object	Ethna_ActionForm	action form֥
+	 *	@var	object	Ethna_ActionForm	ե४֥
 	 */
 	var $action_form;
 
 	/**
-	 *	@var	object	Ethna_ActionForm	action form֥(ά)
+	 *	@var	object	Ethna_ActionForm	ե४֥(ά)
 	 */
 	var $af;
 
