@@ -6,7 +6,7 @@
  *	@author		Masaki Fujimoto <fujimoto@php.net>
  *	@license	http://www.opensource.org/licenses/bsd-license.php The BSD License
  *	@package	Ethna
- *	@version	$Id: Ethna_Controller.php,v 1.15 2004/12/09 05:35:21 fujimoto Exp $
+ *	@version	$Id: Ethna_Controller.php,v 1.24 2004/12/23 06:03:12 fujimoto Exp $
  */
 
 // {{{ Ethna_Controller
@@ -171,12 +171,12 @@ class Ethna_Controller
 	var $i18n = null;
 
 	/**
-	 *	@var	object	Ethna_ActionError	action error֥
+	 *	@var	object	Ethna_ActionError	󥨥顼֥
 	 */
 	var $action_error = null;
 
 	/**
-	 *	@var	object	Ethna_ActionForm	action form֥
+	 *	@var	object	Ethna_ActionForm	ե४֥
 	 */
 	var $action_form = null;
 
@@ -227,17 +227,18 @@ class Ethna_Controller
 				// Smartyץ饰ǥ쥯ȥǻꤹ
 				$tmp = array(SMARTY_DIR . 'plugins');
 				foreach (to_array($value) as $elt) {
-					if ($elt{0} != '/') {
+					if (Ethna_Util::isAbsolute($elt) == false) {
 						$tmp[] = $this->base . (empty($this->base) ? '' : '/') . $elt;
 					}
 				}
 				$this->directory[$key] = $tmp;
 			} else {
-				if ($value{0} != '/') {
+				if (Ethna_Util::isAbsolute($value) == false) {
 					$this->directory[$key] = $this->base . (empty($this->base) ? '' : '/') . $value;
 				}
 			}
 		}
+
 		$this->i18n =& new Ethna_I18N($this->getDirectory('locale'), $this->getAppId());
 		list($this->language, $this->system_encoding, $this->client_encoding) = $this->_getDefaultLanguage();
 		$this->client_type = $this->_getDefaultClientType();
@@ -481,10 +482,10 @@ class Ethna_Controller
 	}
 
 	/**
-	 *	action error֥ȤΥ
+	 *	󥨥顼֥ȤΥ
 	 *
 	 *	@access	public
-	 *	@return	object	Ethna_ActionError	action error֥
+	 *	@return	object	Ethna_ActionError	󥨥顼֥
 	 */
 	function &getActionError()
 	{
@@ -492,10 +493,10 @@ class Ethna_Controller
 	}
 
 	/**
-	 *	action form֥ȤΥ
+	 *	եform֥ȤΥ
 	 *
 	 *	@access	public
-	 *	@return	object	Ethna_ActionForm	action form֥
+	 *	@return	object	Ethna_ActionForm	եform֥
 	 */
 	function &getActionForm()
 	{
@@ -590,74 +591,6 @@ class Ethna_Controller
 		$this->client_type = $client_type;
 	}
 
-	/**
-	 *	ƥץ졼ȥ󥸥(ߤsmartyΤб)
-	 *
-	 *	@access	public
-	 *	@return	object	Smarty	ƥץ졼ȥ󥸥󥪥֥
-	 *	@todo	֥åؿץ饰(etc)б
-	 */
-	function &getTemplateEngine()
-	{
-		$smarty =& new Smarty();
-		$smarty->template_dir = $this->getTemplatedir();
-		$smarty->compile_dir = $this->getDirectory('template_c') . '/tpl_' . md5($smarty->template_dir);
-		if (@is_dir($smarty->compile_dir) == false) {
-			mkdir($smarty->compile_dir, 0755);
-		}
-		$smarty->plugins_dir = $this->getDirectory('plugins');
-
-		// default modifiers
-		$smarty->register_modifier('number_format', 'smarty_modifier_number_format');
-		$smarty->register_modifier('strftime', 'smarty_modifier_strftime');
-		$smarty->register_modifier('count', 'smarty_modifier_count');
-		$smarty->register_modifier('join', 'smarty_modifier_join');
-		$smarty->register_modifier('filter', 'smarty_modifier_filter');
-		$smarty->register_modifier('unique', 'smarty_modifier_unique');
-		$smarty->register_modifier('wordwrap_i18n', 'smarty_modifier_wordwrap_i18n');
-		$smarty->register_modifier('i18n', 'smarty_modifier_i18n');
-		$smarty->register_modifier('checkbox', 'smarty_modifier_checkbox');
-		$smarty->register_modifier('select', 'smarty_modifier_select');
-		$smarty->register_modifier('form_value', 'smarty_modifier_form_value');
-
-		// user defined modifiers
-		foreach ($this->smarty_modifier_plugin as $modifier) {
-			$name = str_replace('smarty_modifier_', '', $modifier);
-			$smarty->register_modifier($name, $modifier);
-		}
-
-		// default functions
-		$smarty->register_function('message', 'smarty_function_message');
-		$smarty->register_function('uniqid', 'smarty_function_uniqid');
-		$smarty->register_function('select', 'smarty_function_select');
-		$smarty->register_function('checkbox_list', 'smarty_function_checkbox_list');
-
-		// user defined functions
-		foreach ($this->smarty_function_plugin as $function) {
-			$name = str_replace('smarty_function_', '', $function);
-			$smarty->register_function($name, $function);
-		}
-
-		// user defined prefilters
-		foreach ($this->smarty_prefilter_plugin as $prefilter) {
-			$smarty->register_prefilter($prefilter);
-		}
-
-		// user defined postfilters
-		foreach ($this->smarty_postfilter_plugin as $postfilter) {
-			$smarty->register_postfilter($postfilter);
-		}
-
-		// user defined outputfilters
-		foreach ($this->smarty_outputfilter_plugin as $outputfilter) {
-			$smarty->register_outputfilter($outputfilter);
-		}
-
-		$this->_setDefaultTemplateEngine($smarty);
-
-		return $smarty;
-	}
-
 	/**
 	 *	ץꥱΥȥݥ
 	 *
@@ -802,9 +735,9 @@ class Ethna_Controller
 		if ($forward_name != null) {
 			$view_name = $this->getViewClassName($forward_name);
 			$view_class =& new $view_name($this->backend, $forward_name, $this->_getForwardPath($forward_name));
+			$view_class->preforward();
 			// ߴ:(
 			if (is_subclass_of($view_class, 'Ethna_ViewClass') == false) {
-				$view_class->preforward();
 				$view_class =& new Ethna_ViewClass($this->backend, $forward_name, $this->_getForwardPath($forward_name));
 			}
 			$view_class->forward();
@@ -845,7 +778,7 @@ class Ethna_Controller
 	 */
 	function _trigger_AMF()
 	{
-		include_once('ethna/contrib/amfphp/app/Gateway.php');
+		include_once('Ethna/contrib/amfphp/app/Gateway.php');
 
 		$this->action_error =& new Ethna_ActionError();
 
@@ -870,7 +803,7 @@ class Ethna_Controller
 	/**
 	 *	顼ϥɥ
 	 *
-	 *	顼ȯɲýԤϤΥ᥽åɤƤӽФ
+	 *	顼ȯɲýԤϤΥ᥽åɤ򥪡С饤ɤ
 	 *	(顼ȥ᡼ݥǥեȤǤϥϻ˥顼ȥ᡼
 	 *	뤬顼ȯ̤˥顼ȥ᡼򤳤
 	 *	뤳Ȥǽ)
@@ -904,283 +837,41 @@ class Ethna_Controller
 		return null;
 	}
 
-	/**
-	 *	ꤵ줿Υե९饹֤̾(֥ȤϹԤʤ)
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	̾
-	 *	@return	string	Υե९饹̾
-	 */
-	function getActionFormName($action_name)
-	{
-		$action_obj =& $this->_getAction($action_name);
-		if (is_null($action_obj)) {
-			return null;
-		}
-
-		return $action_obj['form_name'];
-	}
-
-	/**
-	 *	ꤵ줿Υ饹֤̾(֥ȤϹԤʤ)
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	̾
-	 *	@return	string	Υ饹̾
-	 */
-	function getActionClassName($action_name)
-	{
-		$action_obj =& $this->_getAction($action_name);
-		if ($action_obj == null) {
-			return null;
-		}
-
-		return $action_obj['class_name'];
-	}
-
-	/**
-	 *	ꤵ줿̾бӥ塼饹֤̾(֥ȤϹԤʤ)
-	 *
-	 *	@access	public
-	 *	@param	string	$forward_name	̾
-	 *	@return	string	view classΥ饹̾
-	 */
-	function getViewClassName($forward_name)
-	{
-		if ($forward_name == null) {
-			return null;
-		}
-
-		if (isset($this->forward[$forward_name])) {
-			$forward_obj = $this->forward[$forward_name];
-		} else {
-			$forward_obj = array();
-		}
-
-		if (isset($forward_obj['view_name'])) {
-			$class_name = $forward_obj['view_name'];
-			if (class_exists($class_name)) {
-				return $class_name;
-			}
-		} else {
-			$class_name = null;
-		}
-
-		// viewΥ󥯥롼
-		$this->_includeViewScript($forward_obj, $forward_name);
-
-		if (is_null($class_name) == false && class_exists($class_name)) {
-			return $class_name;
-		} else if (is_null($class_name) == false) {
-			$this->logger->log(LOG_WARNING, 'stated view class is not defined [%s] -> try default', $class_name);
-		}
-
-		$class_name = $this->getDefaultViewClass($forward_name);
-		if (class_exists($class_name)) {
-			return $class_name;
-		} else {
-			$this->logger->log(LOG_DEBUG, 'view class is not defined for [%s] -> use default [%s]', $forward_name, 'Ethna_ViewClass');
-			return 'Ethna_ViewClass';
-		}
-	}
-
-	/**
-	 *	бե९饹̾ά줿Υǥեȥ饹֤̾
-	 *
-	 *	ǥեȤǤ[ץID]_Form_[̾]ȤʤΤǹ߱ƥХ饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	action̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	action form饹̾
-	 */
-	function getDefaultFormClass($action_name, $fallback = true)
-	{
-		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($action_name));
-
-		$r = null;
-		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
-			$r = sprintf("%s_SOAPForm_%s", $this->getAppId(), $postfix);
-		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
-			$tmp = sprintf("%s_MobileAUForm_%s", $this->getAppId(), $postfix);
-			if ($fallback == false || class_exists($tmp)) {
-				$r = $tmp;
-			}
-		}
-
-		if ($r == null) {
-			$r = sprintf("%s_Form_%s", $this->getAppId(), $postfix);
-		}
-		$this->logger->log(LOG_DEBUG, "default action class [%s]", $r);
-		return $r;
-	}
-
-	/**
-	 *	бեѥ̾ά줿Υǥեȥѥ֤̾
-	 *
-	 *	ǥեȤǤ_getDefaultActionPath()Ʊ̤֤(1ե
-	 *	󥯥饹ȥե९饹Ҥ)Τǡߤ˱
-	 *	С饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	action̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	form class륹ץȤΥѥ̾
-	 */
-	function getDefaultFormPath($action_name, $fallback = true)
-	{
-		return $this->getDefaultActionPath($action_name, $fallback);
-	}
-
-	/**
-	 *	б륢󥯥饹̾ά줿Υǥեȥ饹֤̾
-	 *
-	 *	ǥեȤǤ[ץID]_Action_[̾]ȤʤΤǹ߱ƥХ饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	action̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	action class饹̾
-	 */
-	function getDefaultActionClass($action_name, $fallback = true)
-	{
-		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($action_name));
-
-		$r = null;
-		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
-			$r = sprintf("%s_SOAPAction_%s", $this->getAppId(), $postfix);
-		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
-			$tmp = sprintf("%s_MobileAUAction_%s", $this->getAppId(), $postfix);
-			if ($fallback == false || class_exists($tmp)) {
-				$r = $tmp;
-			}
-		}
-
-		if ($r == null) {
-			$r = sprintf("%s_Action_%s", $this->getAppId(), $postfix);
-		}
-		$this->logger->log(LOG_DEBUG, "default action class [%s]", $r);
-		return $r;
-	}
-
-	/**
-	 *	б륢ѥ̾ά줿Υǥեȥѥ֤̾
-	 *
-	 *	ǥեȤǤ"foo_bar" -> "/Foo/Bar.php"ȤʤΤǹ߱ƥС饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$action_name	action̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	action class륹ץȤΥѥ̾
-	 */
-	function getDefaultActionPath($action_name, $fallback = true)
-	{
-		$default_path = preg_replace('/_(.)/e', "'/' . strtoupper('\$1')", ucfirst($action_name)) . '.' . $this->getExt('php');
-		$action_dir = $this->getActiondir();
-
-		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
-			$r = 'SOAP/' . $r;
-		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
-			$r = 'MobileAU/' . $r;
-		} else {
-			$r = $default_path;
-		}
-
-		if ($fallback && file_exists($action_dir . $r) == false && $r != $default_path) {
-			$this->logger->log(LOG_DEBUG, 'client_type specific file not found [%s] -> try defualt', $r);
-			$r = $default_path;
-		}
-
-		$this->logger->log(LOG_DEBUG, "default action path [%s]", $r);
-		return $r;
-	}
-
-	/**
-	 *	̾бӥ塼饹̾ά줿Υǥեȥ饹֤̾
-	 *
-	 *	ǥեȤǤ[ץID]_View_[̾]ȤʤΤǹ߱ƥХ饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$forward_name	forward̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	view class饹̾
-	 */
-	function getDefaultViewClass($forward_name, $fallback = true)
-	{
-		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($forward_name));
-
-		$r = null;
-		if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
-			$tmp = sprintf("%s_MobileAUView_%s", $this->getAppId(), $postfix);
-			if ($fallback == false || class_exists($tmp)) {
-				$r = $tmp;
-			}
-		}
-
-		if ($r == null) {
-			$r = sprintf("%s_View_%s", $this->getAppId(), $postfix);
-		}
-		$this->logger->log(LOG_DEBUG, "default view class [%s]", $r);
-		return $r;
-	}
-
-	/**
-	 *	̾бӥ塼ѥ̾ά줿Υǥեȥѥ֤̾
-	 *
-	 *	ǥեȤǤ"foo_bar" -> "/Foo/Bar.php"ȤʤΤǹ߱ƥС饤ɤ
-	 *
-	 *	@access	public
-	 *	@param	string	$forward_name	forward̾
-	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
-	 *	@return	string	view class륹ץȤΥѥ̾
-	 */
-	function getDefaultViewPath($forward_name, $fallback = true)
-	{
-		$default_path = preg_replace('/_(.)/e', "'/' . strtoupper('\$1')", ucfirst($forward_name)) . '.' . $this->getExt('php');
-		$view_dir = $this->getViewdir();
-
-		if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
-			$r = 'MobileAU/' . $r;
-		} else {
-			$r = $default_path;
-		}
-
-		if ($fallback && file_exists($view_dir . $r) == false && $r != $default_path) {
-			$this->logger->log(LOG_DEBUG, 'client_type specific file not found [%s] -> try defualt', $r);
-			$r = $default_path;
-		}
-
-		$this->logger->log(LOG_DEBUG, "default view path [%s]", $r);
-		return $r;
-	}
-
 	/**
 	 *	¹Ԥ륢֤̾
 	 *
 	 *	@access	private
-	 *	@param	mixed	$default_action_name	action̾
+	 *	@param	mixed	$default_action_name	Υ̾
 	 *	@return	string	¹Ԥ륢̾
 	 */
 	function _getActionName($default_action_name, $fallback_action_name)
 	{
-		// եफ׵ᤵ줿action̾
+		// եफ׵ᤵ줿̾
 		$form_action_name = $this->_getActionName_Form();
 		$form_action_name = preg_replace('/[^a-z0-9\-_]+/i', '', $form_action_name);
 		$this->logger->log(LOG_DEBUG, 'form_action_name[%s]', $form_action_name);
 
+		// Ethnaޥ͡ؤΥեफΥꥯȤϵ
+		if ($form_action_name == "__ethna_info__") {
+			$form_action_name = "";
+		}
+
 		// եफλ̵꤬ϥȥݥȤ˻ꤵ줿ǥեͤѤ
 		if ($form_action_name == "" && count($default_action_name) > 0) {
 			$tmp = is_array($default_action_name) ? $default_action_name[0] : $default_action_name;
+			if ($tmp{strlen($tmp)-1} == '*') {
+				$tmp = substr($tmp, 0, -1);
+			}
 			$this->logger->log(LOG_DEBUG, '-> default_action_name[%s]', $tmp);
 			$action_name = $tmp;
 		} else {
 			$action_name = $form_action_name;
 		}
 
-		// ȥݥȤ󤬻ꤵƤϻʳaction̾ϵݤ
+		// ȥݥȤ󤬻ꤵƤϻʳΥ̾ϵݤ
 		if (is_array($default_action_name)) {
 			if ($this->_isAcceptableActionName($action_name, $default_action_name) == false) {
-				// ʳaction̾ǹä$fallback_action_name(or ǥե)
+				// ʳΥ̾ǹä$fallback_action_name(or ǥե)
 				$tmp = $fallback_action_name != "" ? $fallback_action_name : $default_action_name[0];
 				if ($tmp{strlen($tmp)-1} == '*') {
 					$tmp = substr($tmp, 0, -1);
@@ -1200,15 +891,15 @@ class Ethna_Controller
 	 *
 	 *	ץꥱ˱ƤΥ᥽åɤ򥪡С饤ɤƲ
 	 *	ǥեȤǤ"action_"ǻϤޤեͤ"action_"ʬ
-	 *	("action_sample"ʤ"sample")action̾Ȥưޤ
+	 *	("action_sample"ʤ"sample")̾Ȥưޤ
 	 *
 	 *	@access	protected
-	 *	@return	string	եˤ׵ᤵ줿action̾
+	 *	@return	string	եˤ׵ᤵ줿̾
 	 */
 	function _getActionName_Form()
 	{
 		if (isset($_SERVER['REQUEST_METHOD']) == false) {
-			return $default_action_name;
+			return null;
 		}
 
 		if (strcasecmp($_SERVER['REQUEST_METHOD'], 'post') == 0) {
@@ -1280,7 +971,7 @@ class Ethna_Controller
 
 		if (isset($action_obj['form_name']) == false) {
 			$action_obj['form_name'] = $this->getDefaultFormClass($action_name);
-		} else {
+		} else if (class_exists($action_obj['form_name']) == false) {
 			// ꤵ줿ե९饹ƤʤϷٹ
 			$this->logger->log(LOG_WARNING, 'stated form class is not defined [%s]', $action_obj['form_name']);
 		}
@@ -1314,6 +1005,378 @@ class Ethna_Controller
 		return $retval;
 	}
 
+	/**
+	 *	ե륿
+	 *
+	 *	@access	private
+	 */
+	function _createFilterChain()
+	{
+		$this->filter_chain = array();
+		foreach ($this->filter as $filter) {
+			$file = sprintf("%s/%s.%s", $this->getDirectory('filter'), $filter, $this->getExt('php'));
+			if (file_exists($file)) {
+				include_once($file);
+			}
+			if (class_exists($filter)) {
+				$this->filter_chain[] =& new $filter($this);
+			}
+		}
+	}
+
+	/**
+	 *	̾¹ԵĤƤΤɤ֤
+	 *
+	 *	@access	private
+	 *	@param	string	$action_name			ꥯȤ줿̾
+	 *	@param	array	$default_action_name	ĤƤ륢̾
+	 *	@return	bool	true: false:Ե
+	 */
+	function _isAcceptableActionName($action_name, $default_action_name)
+	{
+		foreach (to_array($default_action_name) as $name) {
+			if ($action_name == $name) {
+				return true;
+			} else if ($name{strlen($name)-1} == '*') {
+				if (strncmp($action_name, substr($name, 0, -1), strlen($name)-1) == 0) {
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+
+	/**
+	 *	ꤵ줿Υե९饹֤̾(֥ȤϹԤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@return	string	Υե९饹̾
+	 */
+	function getActionFormName($action_name)
+	{
+		$action_obj =& $this->_getAction($action_name);
+		if (is_null($action_obj)) {
+			return null;
+		}
+
+		return $action_obj['form_name'];
+	}
+
+	/**
+	 *	бե९饹̾ά줿Υǥեȥ饹֤̾
+	 *
+	 *	ǥեȤǤ[ץID]_Form_[̾]ȤʤΤǹ߱ƥХ饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	ե̾
+	 */
+	function getDefaultFormClass($action_name, $fallback = true)
+	{
+		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($action_name));
+
+		$r = null;
+		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
+			$r = sprintf("%s_SOAPForm_%s", $this->getAppId(), $postfix);
+		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
+			$tmp = sprintf("%s_MobileAUForm_%s", $this->getAppId(), $postfix);
+			if ($fallback == false || class_exists($tmp)) {
+				$r = $tmp;
+			}
+		}
+
+		if ($r == null) {
+			$r = sprintf("%s_Form_%s", $this->getAppId(), $postfix);
+		}
+		$this->logger->log(LOG_DEBUG, "default action class [%s]", $r);
+		return $r;
+	}
+
+	/**
+	 *	getDefaultFormClass()Ǽ饹̾饢̾
+	 *
+	 *	getDefaultFormClass()򥪡С饤ɤ硢碌ƥС饤
+	 *	뤳Ȥ侩(ɬܤǤϤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$class_name		ե९饹̾
+	 *	@return	string	̾
+	 */
+	function actionFormToName($class_name)
+	{
+		$prefix = sprintf("%s_Form_", $this->getAppId());
+		if (preg_match("/$prefix(.*)/", $class_name, $match) == 0) {
+			// ʥ饹̾
+			return null;
+		}
+		$target = $match[1];
+
+		$action_name = substr(preg_replace('/([A-Z])/e', "'_' . strtolower('\$1')", $target), 1);
+
+		return $action_name;
+	}
+
+	/**
+	 *	бեѥ̾ά줿Υǥեȥѥ֤̾
+	 *
+	 *	ǥեȤǤ_getDefaultActionPath()Ʊ̤֤(1ե
+	 *	󥯥饹ȥե९饹Ҥ)Τǡߤ˱
+	 *	С饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	form class륹ץȤΥѥ̾
+	 */
+	function getDefaultFormPath($action_name, $fallback = true)
+	{
+		return $this->getDefaultActionPath($action_name, $fallback);
+	}
+
+	/**
+	 *	ꤵ줿Υ饹֤̾(֥ȤϹԤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@return	string	Υ饹̾
+	 */
+	function getActionClassName($action_name)
+	{
+		$action_obj =& $this->_getAction($action_name);
+		if ($action_obj == null) {
+			return null;
+		}
+
+		return $action_obj['class_name'];
+	}
+
+	/**
+	 *	б륢󥯥饹̾ά줿Υǥեȥ饹֤̾
+	 *
+	 *	ǥեȤǤ[ץID]_Action_[̾]ȤʤΤǹ߱ƥХ饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	󥯥饹̾
+	 */
+	function getDefaultActionClass($action_name, $fallback = true)
+	{
+		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($action_name));
+
+		$r = null;
+		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
+			$r = sprintf("%s_SOAPAction_%s", $this->getAppId(), $postfix);
+		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
+			$tmp = sprintf("%s_MobileAUAction_%s", $this->getAppId(), $postfix);
+			if ($fallback == false || class_exists($tmp)) {
+				$r = $tmp;
+			}
+		}
+
+		if ($r == null) {
+			$r = sprintf("%s_Action_%s", $this->getAppId(), $postfix);
+		}
+		$this->logger->log(LOG_DEBUG, "default action class [%s]", $r);
+		return $r;
+	}
+
+	/**
+	 *	getDefaultActionClass()Ǽ饹̾饢̾
+	 *
+	 *	getDefaultActionClass()򥪡С饤ɤ硢碌ƥС饤
+	 *	뤳Ȥ侩(ɬܤǤϤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$class_name		󥯥饹̾
+	 *	@return	string	̾
+	 */
+	function actionClassToName($class_name)
+	{
+		$prefix = sprintf("%s_Action_", $this->getAppId());
+		if (preg_match("/$prefix(.*)/", $class_name, $match) == 0) {
+			// ʥ饹̾
+			return null;
+		}
+		$target = $match[1];
+
+		$action_name = substr(preg_replace('/([A-Z])/e', "'_' . strtolower('\$1')", $target), 1);
+
+		return $action_name;
+	}
+
+	/**
+	 *	б륢ѥ̾ά줿Υǥեȥѥ֤̾
+	 *
+	 *	ǥեȤǤ"foo_bar" -> "/Foo/Bar.php"ȤʤΤǹ߱ƥС饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$action_name	̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	󥯥饹륹ץȤΥѥ̾
+	 */
+	function getDefaultActionPath($action_name, $fallback = true)
+	{
+		$default_path = preg_replace('/_(.)/e', "'/' . strtoupper('\$1')", ucfirst($action_name)) . '.' . $this->getExt('php');
+		$action_dir = $this->getActiondir();
+
+		if ($this->getClientType() == CLIENT_TYPE_SOAP) {
+			$r = 'SOAP/' . $default_path;
+		} else if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
+			$r = 'MobileAU/' . $default_path;
+		} else {
+			$r = $default_path;
+		}
+
+		if ($fallback && file_exists($action_dir . $r) == false && $r != $default_path) {
+			$this->logger->log(LOG_DEBUG, 'client_type specific file not found [%s] -> try defualt', $r);
+			$r = $default_path;
+		}
+
+		$this->logger->log(LOG_DEBUG, "default action path [%s]", $r);
+		return $r;
+	}
+
+	/**
+	 *	ꤵ줿̾бӥ塼饹֤̾(֥ȤϹԤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$forward_name	̾
+	 *	@return	string	view classΥ饹̾
+	 */
+	function getViewClassName($forward_name)
+	{
+		if ($forward_name == null) {
+			return null;
+		}
+
+		if (isset($this->forward[$forward_name])) {
+			$forward_obj = $this->forward[$forward_name];
+		} else {
+			$forward_obj = array();
+		}
+
+		if (isset($forward_obj['view_name'])) {
+			$class_name = $forward_obj['view_name'];
+			if (class_exists($class_name)) {
+				return $class_name;
+			}
+		} else {
+			$class_name = null;
+		}
+
+		// viewΥ󥯥롼
+		$this->_includeViewScript($forward_obj, $forward_name);
+
+		if (is_null($class_name) == false && class_exists($class_name)) {
+			return $class_name;
+		} else if (is_null($class_name) == false) {
+			$this->logger->log(LOG_WARNING, 'stated view class is not defined [%s] -> try default', $class_name);
+		}
+
+		$class_name = $this->getDefaultViewClass($forward_name);
+		if (class_exists($class_name)) {
+			return $class_name;
+		} else {
+			$this->logger->log(LOG_DEBUG, 'view class is not defined for [%s] -> use default [%s]', $forward_name, 'Ethna_ViewClass');
+			return 'Ethna_ViewClass';
+		}
+	}
+
+	/**
+	 *	̾бӥ塼饹̾ά줿Υǥեȥ饹֤̾
+	 *
+	 *	ǥեȤǤ[ץID]_View_[̾]ȤʤΤǹ߱ƥХ饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$forward_name	forward̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	view class饹̾
+	 */
+	function getDefaultViewClass($forward_name, $fallback = true)
+	{
+		$postfix = preg_replace('/_(.)/e', "strtoupper('\$1')", ucfirst($forward_name));
+
+		$r = null;
+		if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
+			$tmp = sprintf("%s_MobileAUView_%s", $this->getAppId(), $postfix);
+			if ($fallback == false || class_exists($tmp)) {
+				$r = $tmp;
+			}
+		}
+
+		if ($r == null) {
+			$r = sprintf("%s_View_%s", $this->getAppId(), $postfix);
+		}
+		$this->logger->log(LOG_DEBUG, "default view class [%s]", $r);
+		return $r;
+	}
+
+	/**
+	 *	̾бӥ塼ѥ̾ά줿Υǥեȥѥ֤̾
+	 *
+	 *	ǥեȤǤ"foo_bar" -> "/Foo/Bar.php"ȤʤΤǹ߱ƥС饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$forward_name	forward̾
+	 *	@param	bool	$fallback		饤ȼ̤ˤfallback on/off
+	 *	@return	string	view class륹ץȤΥѥ̾
+	 */
+	function getDefaultViewPath($forward_name, $fallback = true)
+	{
+		$default_path = preg_replace('/_(.)/e', "'/' . strtoupper('\$1')", ucfirst($forward_name)) . '.' . $this->getExt('php');
+		$view_dir = $this->getViewdir();
+
+		if ($this->getClientType() == CLIENT_TYPE_MOBILE_AU) {
+			$r = 'MobileAU/' . $r;
+		} else {
+			$r = $default_path;
+		}
+
+		if ($fallback && file_exists($view_dir . $r) == false && $r != $default_path) {
+			$this->logger->log(LOG_DEBUG, 'client_type specific file not found [%s] -> try defualt', $r);
+			$r = $default_path;
+		}
+
+		$this->logger->log(LOG_DEBUG, "default view path [%s]", $r);
+		return $r;
+	}
+
+	/**
+	 *	̾бƥץ졼ȥѥ̾ά줿Υǥեȥѥ֤̾
+	 *
+	 *	ǥեȤǤ"foo_bar"Ȥforward̾"foo/bar" + ƥץ졼ȳĥҤȤʤ
+	 *	Τǹ߱ƥХ饤ɤ
+	 *
+	 *	@access	public
+	 *	@param	string	$forward_name	forward̾
+	 *	@return	string	forwardѥ̾
+	 */
+	function getDefaultForwardPath($forward_name)
+	{
+		return str_replace('_', '/', $forward_name) . '.' . $this->ext['tpl'];
+	}
+	
+	/**
+	 *	ƥץ졼ȥѥ̾̾
+	 *
+	 *	getDefaultForwardPath()򥪡С饤ɤ硢碌ƥС饤
+	 *	뤳Ȥ侩(ɬܤǤϤʤ)
+	 *
+	 *	@access	public
+	 *	@param	string	$forward_path	ƥץ졼ȥѥ̾
+	 *	@return	string	̾
+	 */
+	function forwardPathToName($forward_path)
+	{
+		$forward_path = preg_replace('/^\/+/', '', $forward_path);
+		$forward_path = preg_replace(sprintf('/\.%s$/', $this->getExt('tpl')), '', $forward_path);
+
+		return str_replace('/', '_', $forward_path);
+	}
+
 	/**
 	 *	̾ƥץ졼ȥեΥѥ̾
 	 *
@@ -1332,12 +1395,90 @@ class Ethna_Controller
 		$forward_obj =& $this->forward[$forward_name];
 		if (isset($forward_obj['forward_path']) == false) {
 			// ά
-			$forward_obj['forward_path'] = $this->_getDefaultForwardPath($forward_name);
+			$forward_obj['forward_path'] = $this->getDefaultForwardPath($forward_name);
 		}
 
 		return $forward_obj['forward_path'];
 	}
 
+	/**
+	 *	ƥץ졼ȥ󥸥(ߤsmartyΤб)
+	 *
+	 *	@access	public
+	 *	@return	object	Smarty	ƥץ졼ȥ󥸥󥪥֥
+	 *	@todo	֥åؿץ饰(etc)б
+	 */
+	function &getTemplateEngine()
+	{
+		$smarty =& new Smarty();
+		$smarty->template_dir = $this->getTemplatedir();
+		$smarty->compile_dir = $this->getDirectory('template_c') . '/tpl_' . md5($smarty->template_dir);
+		if (@is_dir($smarty->compile_dir) == false) {
+			mkdir($smarty->compile_dir, 0755);
+		}
+		$smarty->plugins_dir = $this->getDirectory('plugins');
+
+		// default modifiers
+		$smarty->register_modifier('number_format', 'smarty_modifier_number_format');
+		$smarty->register_modifier('strftime', 'smarty_modifier_strftime');
+		$smarty->register_modifier('count', 'smarty_modifier_count');
+		$smarty->register_modifier('join', 'smarty_modifier_join');
+		$smarty->register_modifier('filter', 'smarty_modifier_filter');
+		$smarty->register_modifier('unique', 'smarty_modifier_unique');
+		$smarty->register_modifier('wordwrap_i18n', 'smarty_modifier_wordwrap_i18n');
+		$smarty->register_modifier('i18n', 'smarty_modifier_i18n');
+		$smarty->register_modifier('checkbox', 'smarty_modifier_checkbox');
+		$smarty->register_modifier('select', 'smarty_modifier_select');
+		$smarty->register_modifier('form_value', 'smarty_modifier_form_value');
+
+		// user defined modifiers
+		foreach ($this->smarty_modifier_plugin as $modifier) {
+			$name = str_replace('smarty_modifier_', '', $modifier);
+			$smarty->register_modifier($name, $modifier);
+		}
+
+		// default functions
+		$smarty->register_function('message', 'smarty_function_message');
+		$smarty->register_function('uniqid', 'smarty_function_uniqid');
+		$smarty->register_function('select', 'smarty_function_select');
+		$smarty->register_function('checkbox_list', 'smarty_function_checkbox_list');
+
+		// user defined functions
+		foreach ($this->smarty_function_plugin as $function) {
+			$name = str_replace('smarty_function_', '', $function);
+			$smarty->register_function($name, $function);
+		}
+
+		// user defined prefilters
+		foreach ($this->smarty_prefilter_plugin as $prefilter) {
+			$smarty->register_prefilter($prefilter);
+		}
+
+		// user defined postfilters
+		foreach ($this->smarty_postfilter_plugin as $postfilter) {
+			$smarty->register_postfilter($postfilter);
+		}
+
+		// user defined outputfilters
+		foreach ($this->smarty_outputfilter_plugin as $outputfilter) {
+			$smarty->register_outputfilter($outputfilter);
+		}
+
+		$this->_setDefaultTemplateEngine($smarty);
+
+		return $smarty;
+	}
+
+	/**
+	 *  ƥץ졼ȥ󥸥ΥǥեȾ֤ꤹ
+	 *
+	 *  @access protected
+	 *  @param  object  Smarty  $smarty ƥץ졼ȥ󥸥󥪥֥
+	 */
+	function _setDefaultTemplateEngine(&$smarty)
+	{
+	}
+
 	/**
 	 *	Ѹꤹ
 	 *
@@ -1382,6 +1523,18 @@ class Ethna_Controller
 		return CLIENT_TYPE_WWW;
 	}
 
+	/**
+	 *	ޥ͡㥯饹̾
+	 *
+	 *	@access	public
+	 *	@param	string	$name	ޥ̾͡
+	 *	@return	string	ޥ͡㥯饹̾
+	 */
+	function getManagerClassName($name)
+	{
+		return sprintf('%s_%sManager', $this->getAppId(), ucfirst($name));
+	}
+
 	/**
 	 *	󥹥ץȤ򥤥󥯥롼ɤ
 	 *
@@ -1399,11 +1552,17 @@ class Ethna_Controller
 
 		// class_path°å
 		if (isset($action_obj['class_path'])) {
-			if (file_exists($action_dir . $action_obj['class_path']) == false) {
-				$this->logger->log(LOG_WARNING, 'class_path file not found [%s] -> try default', $action_obj['class_path']);
+			// եѥꥵݡ
+			$tmp_path = $action_obj['class_path'];
+			if (Ethna_Util::isAbsolute($tmp_path) == false) {
+				$tmp_path = $action_dir . $tmp_path;
+			}
+
+			if (file_exists($tmp_path) == false) {
+				$this->logger->log(LOG_WARNING, 'class_path file not found [%s] -> try default', $tmp_path);
 			} else {
-				include_once($action_dir . $action_obj['class_path']);
-				$class_path = $action_obj['class_path'];
+				include_once($tmp_path);
+				$class_path = $tmp_path;
 			}
 		}
 
@@ -1426,14 +1585,20 @@ class Ethna_Controller
 
 		// form_path°å
 		if (isset($action_obj['form_path'])) {
-			if ($action_obj['form_path'] == $class_path) {
+			// եѥꥵݡ
+			$tmp_path = $action_obj['class_path'];
+			if (Ethna_Util::isAbsolute($tmp_path) == false) {
+				$tmp_path = $action_dir . $tmp_path;
+			}
+
+			if ($tmp_path == $class_path) {
 				return;
 			}
-			if (file_exists($action_dir . $action_obj['form_path']) == false) {
-				$this->logger->log(LOG_WARNING, 'form_path file not found [%s] -> try default', $action_obj['form_path']);
+			if (file_exists($tmp_path) == false) {
+				$this->logger->log(LOG_WARNING, 'form_path file not found [%s] -> try default', $tmp_path);
 			} else {
-				include_once($action_dir . $action_obj['form_path']);
-				$form_path = $action_obj['form_path'];
+				include_once($tmp_path);
+				$form_path = $tmp_path;
 			}
 		}
 
@@ -1465,11 +1630,17 @@ class Ethna_Controller
 		$view_dir = $this->getViewdir();
 
 		// view_path°å
-		if (isset($action_obj['view_path'])) {
-			if (file_exists($view_dir . $forward_obj['view_path']) == false) {
-				$this->logger->log(LOG_WARNING, 'view_path file not found [%s] -> try default', $forward_obj['view_path']);
+		if (isset($forward_obj['view_path'])) {
+			// եѥꥵݡ
+			$tmp_path = $forward_obj['view_path'];
+			if (Ethna_Util::isAbsolute($tmp_path) == false) {
+				$tmp_path = $view_dir . $tmp_path;
+			}
+
+			if (file_exists($tmp_path) == false) {
+				$this->logger->log(LOG_WARNING, 'view_path file not found [%s] -> try default', $tmp_path);
 			} else {
-				include_once($action_dir . $forward_obj['view_path']);
+				include_once($tmp_path);
 				return;
 			}
 		}
@@ -1514,62 +1685,6 @@ class Ethna_Controller
 		closedir($dh);
 	}
 
-	/**
-	 *  ƥץ졼ȥ󥸥ΥǥեȾ֤ꤹ
-	 *
-	 *  @access protected
-	 *  @param  object  Smarty  $smarty ƥץ졼ȥ󥸥󥪥֥
-	 */
-	function _setDefaultTemplateEngine(&$smarty)
-	{
-	}
-
-	/**
-	 *	Ethnaޥ͡ꤹ(פʾ϶Υ᥽åɤȤƥС饤ɤƤ褤)
-	 *
-	 *	@access	protected
-	 */
-	function _activateEthnaManager()
-	{
-		$base = dirname(dirname(__FILE__));
-
-		if ($this->config->get('debug') == false) {
-			return;
-		}
-
-		// action
-		$this->action['__ethna_info__'] = array(
-			'form_name' =>	'Ethna_Form_Info',
-			'class_name' =>	'Ethna_Action_Info',
-		);
-		$this->action['__ethna_info_do__'] = array(
-			'form_name' =>	'Ethna_Form_InfoDo',
-			'class_name' =>	'Ethna_Action_InfoDo',
-		);
-
-		// forward
-		$forward_obj = array();
-
-		$forward_obj['forward_path'] = sprintf("%s/tpl/info.tpl", $base);
-		$forward_obj['view_name'] = 'Ethna_Action_Info';
-		$this->forward['__ethna_info__'] = $forward_obj;
-	}
-
-	/**
-	 *	̾бƥץ졼ȥѥ̾ά줿Υǥեȥѥ֤̾
-	 *
-	 *	ǥեȤǤ"foo_bar"Ȥforward̾"foo/bar" + ƥץ졼ȳĥҤȤʤ
-	 *	Τǹ߱ƥХ饤ɤ
-	 *
-	 *	@access	protected
-	 *	@param	string	$forward_name	forward̾
-	 *	@return	string	forwardѥ̾
-	 */
-	function _getDefaultForwardPath($forward_name)
-	{
-		return str_replace('_', '/', $forward_name) . '.' . $this->ext['tpl'];
-	}
-
 	/**
 	 *	եDSNѤǡƹۤ(졼֥ʬ)
 	 *
@@ -1618,44 +1733,34 @@ class Ethna_Controller
 	}
 
 	/**
-	 *	̾¹ԵĤƤΤɤ֤
+	 *	Ethnaޥ͡ꤹ
 	 *
-	 *	@access	private
-	 *	@param	string	$action_name			ꥯȤ줿̾
-	 *	@param	array	$default_action_name	ĤƤ륢̾
-	 *	@return	bool	true: false:Ե
+	 *	פʾ϶Υ᥽åɤȤƥС饤ɤƤ褤
+	 *
+	 *	@access	protected
 	 */
-	function _isAcceptableActionName($action_name, $default_action_name)
+	function _activateEthnaManager()
 	{
-		foreach (to_array($default_action_name) as $name) {
-			if ($action_name == $name) {
-				return true;
-			} else if ($name{strlen($name)-1} == '*') {
-				if (strncmp($action_name, substr($name, 0, -1), strlen($name)-1) == 0) {
-					return true;
-				}
-			}
+		if ($this->config->get('debug') == false) {
+			return;
 		}
-		return false;
-	}
 
-	/**
-	 *	ե륿
-	 *
-	 *	@access	private
-	 */
-	function _createFilterChain()
-	{
-		$this->filter_chain = array();
-		foreach ($this->filter as $filter) {
-			$file = sprintf("%s/%s.%s", $this->getDirectory('filter'), $filter, $this->getExt('php'));
-			if (file_exists($file)) {
-				include_once($file);
-			}
-			if (class_exists($filter)) {
-				$this->filter_chain[] =& new $filter($this);
-			}
-		}
+		include_once(ETHNA_BASE . '/class/Ethna_InfoManager.php');
+
+		// action
+		$this->action['__ethna_info__'] = array(
+			'form_name' =>	'Ethna_Form_Info',
+			'form_path' =>	sprintf('%s/class/Action/Ethna_Action_Info.php', ETHNA_BASE),
+			'class_name' =>	'Ethna_Action_Info',
+			'class_path' =>	sprintf('%s/class/Action/Ethna_Action_Info.php', ETHNA_BASE),
+		);
+
+		// forward
+		$this->forward['__ethna_info__'] = array(
+			'forward_path'	=> sprintf('%s/tpl/info.tpl', ETHNA_BASE),
+			'view_name'		=> 'Ethna_View_Info',
+			'view_path'		=> sprintf('%s/class/View/Ethna_View_Info.php', ETHNA_BASE),
+		);
 	}
 }
 // }}}
