@@ -6,7 +6,7 @@
  *	@author		Masaki Fujimoto <fujimoto@php.net>
  *	@license	http://www.opensource.org/licenses/bsd-license.php The BSD License
  *	@package	Ethna
- *	@version	$Id: Ethna_SOAP_ActoinForm.php,v 1.1 2004/11/29 16:36:27 fujimoto Exp $
+ *	@version	$Id: Ethna_SOAP_ActoinForm.php,v 1.2 2004/12/13 15:08:38 fujimoto Exp $
  */
 
 // {{{ Ethna_SOAP_ActionForm
@@ -39,7 +39,7 @@ class Ethna_SOAP_ActionForm extends Ethna_ActionForm
 	 *	Ethna_SOAP_ActionForm饹Υ󥹥ȥ饯
 	 *
 	 *	@access	public
-	 *	@param	object	Ethna_ActionError	$action_error	action error֥
+	 *	@param	object	Ethna_ActionError	$action_error	󥨥顼֥
 	 */
 	function Ethna_SOAP_ActionForm(&$action_error)
 	{
